/*===========================================================================*/
/*              This supportpac is maintained by Wayne M. Schutz             */
/*                                                                           */
/*              Chris Petty is the original author.                          */
/*                                                                           */
/*  (C) Copyright IBM Corporation 1995, 1997, 1998, 1999, 2000, 2002, 2003   */
/*===========================================================================*/
/*                                                                           */
/*  See the read.me file for change history                                  */
/*                                                                           */
/*===========================================================================*/
/* Module Name: saveqmgr.c                                                   */
/*===========================================================================*/
/*                                                                           */
/* This is a program to inquire of a default queue manager about its         */
/* own attributes and those of all the objects, (queues, channels and        */
/* processes), defined to it. The inquiry may be made of a local or remote   */
/* queue manager.                                                            */
/*                                                                           */
/* The program takes this information and appends it to a file               */
/* SAVEQMGR.TST which is of a format suitable for RUNMQSC. It could,         */
/* therefore, be used to recreate or clone a queue manager.                  */
/*                                                                           */
/* It is offered as an example of using Programmable Command Formats (PCFs)  */
/* as a method for administering a queue manager.                            */
/*                                                                           */
/* The program puts a message to the SYSTEM.ADMIN.COMMAND.QUEUE at the       */
/* queue manager specified by the -r flag. If this flag is not specified     */
/* the default local queue manager is assumed. The reply is sent to          */
/* a dynamic queue at the queue manager specified by the -m flag, again      */
/* the default queue manager is assumed if this is not specified. Access to  */
/* a remote queue manager is through the transmission queue of the same      */
/* name as the queue manager supplied. It is left to the user to set up the  */
/* MQSeries network appropriately.                                           */
/*                                                                           */
/* The request messages are set to timeout if not processed within three     */
/* minutes, and are non-persitient. The GET process handling the replies     */
/* will also wait for a maximum of three minutes before terminating.         */
/* This prevents administration or transmission queues filling up if         */
/* connections to various nodes are not available or the command server      */
/* at the destination queue manager is not running.                          */
/*===========================================================================*/


/* Include standard libraries */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/* Include MQSeries headers */
#include <cmqc.h>
#include <cmqcfc.h>
#include <cmqxc.h>

/* Include Application headers */
#include "saveqmgr.h"

#define USAGE \
"Usage is:\n saveqmgr -v ver -m LclQMgr -r RmtQMgr -f|F <filename> -c -s -n -q -R -p -i\n\n" \
" -v determines which version of MQSC to generate\n" \
"      and can be '1','2','5', '51', '52' or '53'\n" \
"      1 = MQ for Windows, 2, 5, 51, 52, 53 refer to normal Websphere MQ servers\n" \
"    Note: 5, 51, 52, 53 are only available if the program has been \n" \
"      compiled with the correct level of the queue manager.\n" \
" -m is the name of the local qmgr to connect (MQCONN)\n" \
" -r is the name of the remote qmgr (XMITQ name)\n" \
" -f allows the output file to be named, \n" \
"    if -f is not specified, the output file defaults to SAVEQMGR.TST\n" \
"    if -f is specified without a filename, it is named \"qmgrname\".MQS.\n" \
" -F behaves the same as -f except no header line is created in the output file\n"\
" -c directs output to STDOUT and overrides use of -f flag\n" \
" -s suppress creation of objects that begin with \"SYSTEM*\"\n" \
" -n directs MQGET to be issued without a MQGMO_CONVERT option\n" \
" -q Quiet mode, do not issue warning about unknown PCF attributes\n" \
" -R create RESET CHANNEL commands for non-client/svrconn channels\n" \
" -p Create 'local queue' definitions from PERMDYN queues.\n" \
" -i Ignore 'pcf' error code 'damaged object' (2101).\n" \
" All parameters are optional."

int convert = 1;                    /* determines whether to issue MQGMO_CONVERT */
int noisy = 1;                      /* whether or not to put bad PCF attr messages */
int chlReset = 0;                   /* determines whether or not to issue RESET CHANNEL cmds */
int header=1;                       /* whether to create header in output file */ /* v513 */
int permDyn=0;         /* v530 */   /* whether to create qlocal from Permdyn */
int ignoreDamaged=0;   /* v530 */   /* whether to ignore damaged obj messages */

int main( int argc, char *argv[] )
{
  MQCHAR   LclQMgrName[MQ_Q_MGR_NAME_LENGTH+1];      /* Name of connected queue mgr     */
  MQCHAR   RmtQMgrName[MQ_Q_MGR_NAME_LENGTH+1];      /* Name of QMgr to inquire of      */
  MQCHAR48 ReplyToQ;         /* DynamicQName for this run       */

  MQHCONN  hConn=MQHC_UNUSABLE_HCONN;   /* handle to connected queue mgr   */
  MQLONG   CompCode=0;         /* MQ API completion code          */
  MQLONG   Reason=0;           /* Reason qualifying above         */

  MQHOBJ   hAdminQ=MQHO_UNUSABLE_HOBJ; /* handle to output queue          */
  MQHOBJ   hReplyQ=MQHO_UNUSABLE_HOBJ; /* handle to input queue           */

  int      ParmCount = 0;
  int      UserVersion =  0;      /* version number user requested */
  int      QmgrVersion =  0;      /* version number we detected */
  int      SysObjs = 1;         /* [v51].. create "system" objects */

  MQLONG   ObjectSets=0;       /* number of sets of objects to process */

  MQLONG   RetCode = 0;        /* Return code from this program */


  /* V3.2 filename constructed from queue manager name or "saveqmgr.tst" */
  char fileName[FILENAMESIZE+1];

   /* file handle for output file [v51] */
   FILE *outputfp = 0;

  /* ********************************************************************** */
  /* Get a connection to the queue manager and open the                     */
  /* required application queues.                                           */
  /*                                                                        */
  /* SYSTEM.ADMIN.COMMAND.QUEUE is the output queue for this application    */
  /* and the default command queue for the queue manager. Messages placed   */
  /* on this queue must be of a format suitable for processing by the       */
  /* supplied MQSeries command server, STRMQCSV.                            */
  /*                                                                        */
  /* SAVEQMGR.* is the prefix for the name of the dynamic queue to which    */
  /* the command server will send the messages generated in response to     */
  /* our request. This will be based on the definition of the queue         */
  /* SYSTEM.DEFAULT.MODEL.QUEUE                                             */
  /* ********************************************************************** */

  /* Process input parameter list                                           */

  /* Set queue manager names to defaults */
  memset( LclQMgrName, '\0', sizeof( LclQMgrName ) );
  memset( RmtQMgrName, '\0', sizeof( RmtQMgrName ) );

  strcpy(fileName, DEFAULTOUTPUTFILE ); /* V3.2 assume old output name */

  ParmCount = 1;

  while ( argc > ParmCount ) {

     if ( ParmCount < argc-1 && memcmp(argv[ParmCount], "-m", 2 ) == 0 ) {
        strncpy( LclQMgrName, argv[ParmCount+1], MQ_Q_MGR_NAME_LENGTH );
        ParmCount += 2;
     } else if ( ParmCount < argc-1 && memcmp(argv[ParmCount], "-r", 2 ) == 0 ) {
        strncpy( RmtQMgrName, argv[ParmCount+1], MQ_Q_MGR_NAME_LENGTH );
        ParmCount += 2;
     } else if ( ParmCount < argc-1 && memcmp(argv[ParmCount], "-v", 2 ) == 0 ) {
        UserVersion = atoi( argv[ParmCount+1] );
        ParmCount += 2;
     } else if ( memcmp(argv[ParmCount], "-f", 2 ) == 0  ||
                 memcmp(argv[ParmCount], "-F", 2 ) == 0 ) {  /* v513 */
        if ( memcmp(argv[ParmCount], "-F", 2 ) == 0 ) header=0; /* v513 */
        fileName[0] = '\0'; /* V3.2 user wants unique output queue names */
        ParmCount += 1;
        if (ParmCount < argc && argv[ParmCount][0] != '-'){ /* is the next token not a switch ? */
        /* yes.. then the next token is a filename */
           strncpy(fileName, argv[ParmCount], FILENAMESIZE);
           ParmCount += 1;
          } /* End if next char is not a switch */
     } else if ( memcmp( argv[ParmCount], "-c" ,2 ) == 0 ) {  /* [v51] stdout support */
        outputfp = stdout;   /* user wants stdout option */
        ParmCount += 1;
     } else if ( memcmp( argv[ParmCount], "-s" ,2 ) == 0 ) {
        SysObjs = 0;   /* user does NOT want SYSTEM objects */
        ParmCount += 1;
     } else if ( memcmp( argv[ParmCount], "-n" ,2 ) == 0 ) {
        convert = 0;   /* user does NOT want MQGMO_CONVERT  */
        ParmCount += 1;
     } else if ( memcmp( argv[ParmCount], "-R" ,2 ) == 0 ) {
        chlReset = 1;   /* user wants to create channel reset commands */
        ParmCount += 1;
     } else if ( memcmp( argv[ParmCount], "-q" ,2 ) == 0 ) {
        noisy = 0;   /* user does NOT want warning msgs   */
        ParmCount += 1;
     } else if ( memcmp( argv[ParmCount], "?", 1 ) == 0 ) {
        fprintf(stderr, USAGE);
        fprintf(stderr, " \n\n %s\n\n", THISVERSION);
        exit( ERROR );
     } else if ( memcmp( argv[ParmCount], "-?", 2 ) == 0 ) {
        fprintf(stderr, USAGE);
        fprintf(stderr, " \n\n %s\n\n", THISVERSION);
        exit( ERROR );
     } else if ( memcmp( argv[ParmCount], "-h", 2 ) == 0 ) {
        fprintf(stderr, USAGE);
        fprintf(stderr, " \n\n %s\n\n", THISVERSION);
        exit( ERROR );
     } else if ( memcmp( argv[ParmCount], "-p" ,2 ) == 0 ) {
        permDyn = 1;   /* v530 user wants qlocals created from permdyn */
        ParmCount += 1;
     } else if ( memcmp( argv[ParmCount], "-i" ,2 ) == 0 ) {
        ignoreDamaged = 1;   /* v530 user wants to ignore damaged objects */
        ParmCount += 1;

     } else {
        /* invalid parameter */
        fprintf(stderr, "Invalid invocation\n" );
        fprintf(stderr, USAGE);
        fprintf(stderr, " \n\n %s\n\n", THISVERSION);
        exit( ERROR );
     } /* endif */


  } /* endwhile */

/* A little hack to print which version of MQSeries we were */
/* compiled for .... */

fprintf(stderr, "%s\n", THISVERSION);
#if defined (MQCMD_INQUIRE_AUTH_INFO)
  fprintf (stderr, "Compiled for Websphere MQ V5.3 on %s\n",__DATE__);
#elif defined (MQCMDL_LEVEL_520)
  fprintf (stderr, "Compiled for MQSeries V5.2 on %s\n",__DATE__);
#elif defined (MQOO_BIND_ON_OPEN)
  fprintf (stderr, "Compiled for MQSeries V5.1 on %s\n",__DATE__);
#elif defined (MQDL_NOT_SUPPORTED)
  fprintf (stderr, "Compiled for MQSeries V5.0 on %s\n",__DATE__);
#else
  fprintf (stderr, "Compiled for MQSeries V2 on %s\n",__DATE__);
#endif

        switch (UserVersion)
          {
            case 0:
             /* user didn't specify a level , leave alone for now */
              break;
            case 1:
              UserVersion = MQCMDL_LEVEL_110;
              break;
            case 2:
              UserVersion = MQCMDL_LEVEL_200;
              break;
/* don't allow these to be specified if not compiled for this level */
#if defined (MQDL_NOT_SUPPORTED)
            case 5:
              UserVersion = MQCMDL_LEVEL_500;
              break;
#endif
/* don't allow these to be specified if not compiled for this level */
#if defined (MQOO_BIND_ON_OPEN)
            case 51:
              UserVersion = MQCMDL_LEVEL_510;
              break;
#endif
/* don't allow these to be specified if not compiled for this level */
#if defined (MQCMDL_LEVEL_520)
            case 52:
              UserVersion = MQCMDL_LEVEL_520;
              break;
#endif
#if defined (MQCMD_INQUIRE_AUTH_INFO)
            case 53:
              UserVersion = MQCMDL_LEVEL_530;
              break;
#endif
            default:
              fprintf(stderr,
                      "Invalid Version Number %d\n", UserVersion);
              fprintf(stderr, USAGE);
              exit ( ERROR );
              break;
          }/* End switch*/


  MQCONN(  LclQMgrName                   /* I  : use default queue manager  */
        , &hConn                         /*  O : queue manager handle       */
        , &CompCode                      /*  O : Completion code            */
        , &Reason                        /*  O : Reason qualifying CompCode */
        );

  /* v3.2 check for failed, since os/400 can return a warning .... */

  if ( CompCode == MQCC_FAILED ) {
     fprintf(stderr, "MQCONN failed for %s, CC=%ld RC=%ld\n"
           , LclQMgrName
           , CompCode
           , Reason
           );
     exit( ERROR );
  } /* endif */


  CompCode = OpenQ( hConn
                 , RmtQMgrName
                 , "SYSTEM.ADMIN.COMMAND.QUEUE\0"
                 , NULL
                 , MQOO_OUTPUT
                 , &hAdminQ
                 );

  if (CompCode!=MQCC_FAILED) {
      strcpy( ReplyToQ, "SAVEQMGR.*" );

      CompCode = OpenQ( hConn
                     , LclQMgrName
                     , "SYSTEM.DEFAULT.MODEL.QUEUE\0"
                     , &ReplyToQ
                     , MQOO_INPUT_EXCLUSIVE
                     , &hReplyQ
                     );
  }/* End if*/

  if (CompCode==MQCC_FAILED){
     MQDISC ( &hConn,
              &CompCode,
              &Reason
             );
     exit ( ERROR );

  }

  /* ******************************************************************** */
  /* QUEUE MANAGER                                                        */
  /* First, get the command level of the queue manager. We need this to   */
  /* tell us what further objects (such as namelists) we can query.       */
  /* ******************************************************************** */

  fprintf(stderr, "Requesting attributes of the queue manager...\n" );

  RetCode = InquireDefinition( hConn          /* QMgr connection handle    */
                   , hAdminQ                 /* Handle to open output Q   */
                   , ReplyToQ                /* Name of queue to reply to */
                   , MQCMD_INQUIRE_Q_MGR     /* Command to execute        */
                   , 0                       /* Attribute to search on    */
                   , 0                       /* Max.length of search attr */
                   , NULL                    /* Scope of search           */
                   , 1L                      /* Length of search string   */
                   );

  if (RetCode!=ERROR) {
      RetCode =  ProcessReplyQ( hConn    /* QMgr connection handle    */
                     , hReplyQ           /* Handle to open input Q    */
                     , &UserVersion       /* Command level user is requesting */
                     , &QmgrVersion      /* Command level we detected        */
                     , &outputfp         /* [v51]output file handle */
                     , fileName          /* v32 Name of the output file  */
                     , 1                 /* number of reply sets to process */
                     , SysObjs           /* system object flag [v51]  */
                     );
  }/* End if*/

  /* ******************************************************************** */
  /* Other objects:                                                       */
  /* Now we can request all the other objects that are appropriate for    */
  /* this level of the queue manager.  For example, mq/win doesn't        */
  /* support processes or namelists                                       */
  /* ******************************************************************** */

  ObjectSets = 0;                           /* number of sets requested   */


#if defined (MQCMD_INQUIRE_AUTH_INFO)

   if (QmgrVersion >= MQCMDL_LEVEL_530 &&    /* We're V5.3 or better */
       UserVersion >= MQCMDL_LEVEL_530       /* and user wants authinfo */
       ){

       /* ******************************************************************** */
       /* [v53] Authinfo Objects                                                       */
       /* ******************************************************************** */

      if (RetCode!=ERROR) {
           ObjectSets++;
           fprintf(stderr, "Requesting attributes of all authinfo objects...\n" );
           RetCode = InquireDefinition( hConn         /* QMgr connection handle    */
                          , hAdminQ                 /* Handle to open output Q   */
                          , ReplyToQ                /* Name of queue to reply to */
                          , MQCMD_INQUIRE_AUTH_INFO /* Command to execute        */
                          , MQCA_AUTH_INFO_NAME       /* Attribute to search on    */
                          , MQ_AUTH_INFO_NAME_LENGTH  /* Max.length of search attr */
                          , "*"                     /* Scope of search           */
                          , 1L                      /* Length of search string   */
                          );
          }/* End if retcode != ERROR */
         }
#endif

  if (RetCode!=ERROR) {
     ObjectSets++;
     fprintf(stderr, "Requesting attributes of all queues...\n" );
     RetCode = InquireDefinition( hConn         /* QMgr connection handle    */
                      , hAdminQ                 /* Handle to open output Q   */
                      , ReplyToQ                /* Name of queue to reply to */
                      , MQCMD_INQUIRE_Q         /* Command to execute        */
                      , MQCA_Q_NAME             /* Attribute to search on    */
                      , MQ_Q_NAME_LENGTH        /* Max.length of search attr */
                      , "*"                     /* Scope of search           */
                      , 1L                      /* Length of search string   */
                      );
  }/* End if retcode != ERROR */

  if (RetCode!=ERROR) {
     ObjectSets++;
     fprintf(stderr, "Requesting attributes of all channels...\n" );
     RetCode = InquireDefinition( hConn         /* QMgr connection handle    */
                      , hAdminQ                 /* Handle to open output Q   */
                      , ReplyToQ                /* Name of queue to reply to */
                      , MQCMD_INQUIRE_CHANNEL   /* Command to execute        */
                      , MQCACH_CHANNEL_NAME     /* Attribute to search on    */
                      , MQ_CHANNEL_NAME_LENGTH  /* Max.length of search attr */
                      , "*"                     /* Scope of search           */
                      , 1L                      /* Length of search string   */
                      );
  }/* End if retcode != ERROR */

  /* Special note.  Contrary to the documentation I have, it appears that */
  /* V5.1 of MQSeries includes client-conn channels when you ask for all  */
  /* channels.  Since we asked for all channel types above, we will not do */
  /* it again here ...                                                    */

  if (QmgrVersion > MQCMDL_LEVEL_110 &&  /* MQ/Win doesn't support clntconn */
      QmgrVersion < MQCMDL_LEVEL_510 &&  /* see BIG note above            */
      UserVersion > MQCMDL_LEVEL_110 &&  /* and user wants client-conns   */
      MQAT_DEFAULT != MQAT_OS400 ) {     /* Os/400 doesn't support clients*/

    if (RetCode!=ERROR) {
       ObjectSets++;
       RetCode = InquireDefinition( hConn         /* QMgr connection handle    */
                        , hAdminQ                 /* Handle to open output Q   */
                        , ReplyToQ                /* Name of queue to reply to */
                        , XZCMD_INQUIRE_CHANNEL_CLNT   /* Command to execute        */
                        , MQCACH_CHANNEL_NAME     /* Attribute to search on    */
                        , MQ_CHANNEL_NAME_LENGTH  /* Max.length of search attr */
                        , "*"                     /* Scope of search           */
                        , 1L                      /* Length of search string   */
                        );
      }/* End if retcode != ERROR */
   }


  if (QmgrVersion > MQCMDL_LEVEL_110 &&  /* Not MQ/Win */
      UserVersion > MQCMDL_LEVEL_110     /* and user wants processes      */
   ) {

     if (RetCode!=ERROR) {
        ObjectSets++;
        fprintf(stderr, "Requesting attributes of all processes...\n" );
        RetCode = InquireDefinition( hConn         /* QMgr connection handle    */
                         , hAdminQ                 /* Handle to open output Q   */
                         , ReplyToQ                /* Name of queue to reply to */
                         , MQCMD_INQUIRE_PROCESS   /* Command to execute        */
                         , MQCA_PROCESS_NAME       /* Attribute to search on    */
                         , MQ_PROCESS_NAME_LENGTH  /* Max.length of search attr */
                         , "*"                     /* Scope of search           */
                         , 1L                      /* Length of search string   */
                         );
      }/* End if retcode != ERROR */

    }

#if defined (MQCMD_INQUIRE_NAMELIST)

   if (QmgrVersion >= MQCMDL_LEVEL_510 &&    /* We're V5.1 or better */
       UserVersion >= MQCMDL_LEVEL_510       /* and user wants namelists */
       ){

       /* ******************************************************************** */
       /* [v51] Namelists                                                       */
       /* ******************************************************************** */

      if (RetCode!=ERROR) {
           ObjectSets++;
           fprintf(stderr, "Requesting attributes of all namelists...\n" );
           RetCode = InquireDefinition( hConn         /* QMgr connection handle    */
                          , hAdminQ                 /* Handle to open output Q   */
                          , ReplyToQ                /* Name of queue to reply to */
                          , MQCMD_INQUIRE_NAMELIST  /* Command to execute        */
                          , MQCA_NAMELIST_NAME       /* Attribute to search on    */
                          , MQ_NAMELIST_NAME_LENGTH  /* Max.length of search attr */
                          , "*"                     /* Scope of search           */
                          , 1L                      /* Length of search string   */
                          );
          }/* End if retcode != ERROR */
         }
#endif


  /* ******************************************************************** */
  /* Now that we have requested all the required object attributes,       */
  /* call ProcessReplyQ to handle the responses.  ObjectSets tell us      */
  /* how many sets of replies there are.                                  */
  /* ******************************************************************** */

  if (RetCode!=ERROR) {
      RetCode =  ProcessReplyQ( hConn    /* QMgr connection handle    */
               , hReplyQ                     /* Handle to open input Q    */
               , &UserVersion          /* Command level user is requesting */
               , &QmgrVersion         /* Command level we detected        */
               , &outputfp            /* [v51]output file handle */
               , fileName            /* v32 Name of the output file  */
               , ObjectSets          /* number of sets of replies to process */
               , SysObjs           /* system object flag [v51]  */
               );
  }

  /* ******************************************************************** */
  /* Close down in an orderly manner.                                     */
  /*                                                                      */
  /* No need to test for these calls failing, the result is the same!     */
  /* ******************************************************************** */
  MQCLOSE(  hConn                            /* Queue manager handle      */
         , &hAdminQ                          /* Handle to queue           */
         ,  MQCO_NONE                        /* Parameters on close       */
         , &CompCode                         /* Completion code           */
         , &Reason                           /* Reason code               */
         );

  MQCLOSE(  hConn                            /* Queue manager handle      */
         , &hReplyQ                          /* Handle to queue           */
         ,  MQCO_NONE                        /* Parameters on close       */
         , &CompCode                         /* Completion code           */
         , &Reason                           /* Reason code               */
         );

  /* [v51] close output file */
  if (outputfp) fclose (outputfp);


 if (hConn!=MQHC_UNUSABLE_HCONN) {
     MQDISC( &hConn
           , &CompCode
           , &Reason
           );
 }/* End if*/

   return( RetCode );
} /* end MAIN */

/* ************************************************************************ */
/* InquireDefinition()                                                      */
/*                                                                          */
/* This is a procedure to inquire of the queue manager about the objects    */
/* defined to it.                                                           */
/*                                                                          */
/* SearchType indicates the type of object for which information is         */
/* required. Objects can either be queues, channels, processes or the       */
/* queue manager itself.                                                    */
/*                                                                          */
/* SearchParm indicates the parameter on which the request is based. A      */
/* SearchString specifies a 'mask' for the search. So, for example, a       */
/* SearchParm of MQCA_Q_NAME and a SearchString of "*" will request         */
/* information about all queues.                                            */
/*                                                                          */
/* SearchParmLen indicates the maximum length of a string of this type,     */
/* SearchStringLen indicates the length of the search mask. Note that       */
/* NULL characters in the SearchString are treated as valid. The            */
/* SearchString should, therefore, be padded with blanks (' ').             */
/*                                                                          */
/*                                                                          */
/* The actual request consists of a Request Header and a parameter block    */
/* used to specify the context for the search. The header and the parameter */
/* block follow each other in a contiguous buffer which is pointed  to by   */
/* the variable pAdminMsg. This entire buffer is then put to  the queue.    */
/*                                                                          */
/* The command server, (use STRMQCSV to start it), processes the            */
/* SYSTEM.ADMIN.COMMAND.QUEUE and puts a reply on the application           */
/* ReplyToQ for each defined queue.                                         */
/* ************************************************************************ */
MQLONG InquireDefinition( MQHCONN   hConn
                      , MQHOBJ    hAdminQ
                      , MQCHAR48  ReplyToQ
                      , MQLONG    SearchType /* QUEUE|CHANNEL|PROCESS|QMGR */
                      , MQLONG    SearchParm
                      , MQLONG    SearchParmLen
                      , MQCHAR   *SearchString
                      , MQLONG    SearchStringLen
                      )
{
  MQLONG               AdminMsgLen;      /* Length of user message buffer   */
  MQBYTE              *pAdminMsg;        /* Ptr to outbound data buffer     */
  MQCFH               *pPCFHeader;       /* Ptr to PCF header structure     */
  MQCFST              *pPCFString;       /* Ptr to PCF string parm block    */
  MQCFIN                      *pPCFInteger;          /* v32 ptr to chltype param for clntconn */
  MQLONG              RetCode = 0;       /* Return code from function */
  /* ----------------------------------------------------------------------- */
  /* The search for queue manager attributes is slightly different to that   */
  /* for other objects, in that there is only one.                           */
  /* The PCF header is the only piece of the request message required in     */
  /* this case.                                                              */
  /*                                                                         */
  /* When requesting attributes for queues, channels or processes a          */
  /* parameter block follows the header in which you specify the context     */
  /* of the request. That is, you specify the parameter to base the          */
  /* request on, and a mask to search against. In this example we want       */
  /* all attributes for all objects so we base our search on object name     */
  /* and use the '*' wildcard to indicate a generic search.                  */
  /*                                                                         */
  /* This procedure will eventually perform an MQPUT to the                  */
  /* SYSTEM.ADMIN.COMMAND.QUEUE.  The request outlined above will be         */
  /* encapsulated in a single message occupying a contiguous block of        */
  /* storage. To help with manipulation of this buffer we will use           */
  /* several pointers into the buffer.                                       */
  /*                                                                         */
  /* pAdminMsg points to the start of the entire message buffer.             */
  /* This is a pointer to a flat block of storage allocated to be large      */
  /* enough to contain the entire request.                                   */
  /*                                                                         */
  /* pPCFHeader points to a structure of type MQCFH, a command format        */
  /* header. It is points to the same address as pAdminMsg and               */
  /* effectively remaps the first piece of our message buffer. It is         */
  /* used to indicate the type of command we wish to execute and the         */
  /* number of parameter blocks following in the message buffer.             */
  /*                                                                         */
  /* pPCFString points to a structure of type MQCFST, a command format       */
  /* string. It points into the message buffer at an address immediately     */
  /* following the header described above. It is used to map the following   */
  /* bytes onto a PCF string parameter block which in this case is used      */
  /* to indicate the name of the object we want details about, * indicating  */
  /* all.                                                                    */
  /*                                                                         */
  /* Note that this example is a generic search for all attributes of        */
  /* all objects known to the queue manager. By using different,             */
  /* or more, parameter blocks in the request header it is possible          */
  /* to narrow the search.                                                   */
  /* ----------------------------------------------------------------------- */
  if ( SearchType == MQCMD_INQUIRE_Q_MGR ) {
     /* Inquire of the queue manager attributes. It is only necessary */
     /* to complete the request header in this case.                  */

     /* Set the length for the message buffer */
     AdminMsgLen = MQCFH_STRUC_LENGTH;

     /* Allocate storage for the message buffer and set a pointer to */
     /* its start address.                                           */
     pAdminMsg   = (MQBYTE *)malloc( AdminMsgLen );
     memset( pAdminMsg, '\0', AdminMsgLen );

     /* pPCFHeader is set equal to pAdminMsg in order to provide a */
     /* structure to the newly allocated block of storage. We can  */
     /* then create a request header of the correct format in the  */
     /* message buffer.                                            */
     pPCFHeader  = (MQCFH *)pAdminMsg;

     /* Setup request header */
     pPCFHeader->Type           = MQCFT_COMMAND;
     pPCFHeader->StrucLength    = MQCFH_STRUC_LENGTH;
     pPCFHeader->Version        = MQCFH_VERSION_1;
     pPCFHeader->Command        = SearchType;
     pPCFHeader->MsgSeqNumber   = MQCFC_LAST;
     pPCFHeader->Control        = MQCFC_LAST;
     pPCFHeader->ParameterCount = 0;

  } else {
     /* Inquire of queue, channel or process attributes; exact one to */
     /* be indicated by SearchType.                                   */

     /* Set the length for the message buffer */
         AdminMsgLen = MQCFH_STRUC_LENGTH
                 + MQCFST_STRUC_LENGTH_FIXED + SearchParmLen
                                 ;

     /* Allocate storage for the message buffer and set a pointer to */
     /* its start address.                                           */
     pAdminMsg   = (MQBYTE *)malloc( AdminMsgLen );
     memset( pAdminMsg, '\0', AdminMsgLen );

     /* pPCFHeader is set equal to pAdminMsg in order to provide a */
     /* structure to the newly allocated block of storage. We can  */
     /* then create a request header of the correct format in the  */
     /* message buffer.                                            */
     pPCFHeader  = (MQCFH *)pAdminMsg;

     /* pPCFString is set to point into the message buffer immediately */
     /* after the request header. This allows us to specify a context  */
     /* for the request in the required format.                        */
     pPCFString  = (MQCFST *)(pAdminMsg
                             + MQCFH_STRUC_LENGTH
                             );

     /* Setup request header */
     pPCFHeader->Type           = MQCFT_COMMAND;
     pPCFHeader->StrucLength    = MQCFH_STRUC_LENGTH;
     pPCFHeader->Version        = MQCFH_VERSION_1;
     pPCFHeader->Command        = SearchType;
     pPCFHeader->MsgSeqNumber   = MQCFC_LAST;
     pPCFHeader->Control        = MQCFC_LAST;
     pPCFHeader->ParameterCount = 1;

     /* Setup parameter block */
     pPCFString->Type           = MQCFT_STRING;
     pPCFString->StrucLength    = MQCFST_STRUC_LENGTH_FIXED + SearchParmLen;
     pPCFString->Parameter      = SearchParm;
     pPCFString->CodedCharSetId = MQCCSI_DEFAULT;
     pPCFString->StringLength   = SearchParmLen;
     memset( pPCFString->String, ' ', SearchParmLen );
     strncpy( pPCFString->String, SearchString, SearchStringLen );
  } /* endif */

  /* v32 More additions for clntconn channels */
  /* V32 add integer attribute chltype to query   */
  if (SearchType == XZCMD_INQUIRE_CHANNEL_CLNT)
  {
        /* V32 change some previously set values */
        AdminMsgLen += MQCFIN_STRUC_LENGTH;
        pAdminMsg   = (MQBYTE *) realloc( (void *) pAdminMsg, AdminMsgLen );
        pPCFHeader  = (MQCFH *)pAdminMsg;

        /*  V32  memset( pAdminMsg, '\0', AdminMsgLen ); */
        pPCFHeader->Command        = MQCMD_INQUIRE_CHANNEL;
        pPCFHeader->ParameterCount = 2;

        /* V32 fill in values for clntconn channel type */
        pPCFInteger = (MQCFIN *)(pAdminMsg
                             + MQCFH_STRUC_LENGTH
                             + MQCFST_STRUC_LENGTH_FIXED
                             + SearchParmLen
                             );

        pPCFInteger->Type        = MQCFT_INTEGER;
        pPCFInteger->StrucLength = MQCFIN_STRUC_LENGTH;
        pPCFInteger->Parameter   = MQIACH_CHANNEL_TYPE;
        pPCFInteger->Value       = MQCHT_CLNTCONN;
  }

  /* V32 End Clntconn addition */

  /* Put the newly created message to the input queue for the */
  /* MQSeries command server                                  */
  RetCode = PutMsg(  hConn
                  ,  MQFMT_ADMIN
                  ,  hAdminQ
                  , "SYSTEM.ADMIN.COMMAND.QUEUE\0"
                  ,  ReplyToQ
                  ,  (MQBYTE *)pAdminMsg
                  ,  AdminMsgLen
                  );

  /* Free the storage allocated to the message buffer */
  if (pAdminMsg) free( pAdminMsg );

  return (RetCode);

}

/* ************************************************************************ */
/* ProcessReplyQ()                                                          */
/*                                                                          */
/* The MQSeries command server has sent replies to our previous request     */
/* to the applications ReplyToQ. We are passed a handle to the open         */
/* queue, hReplyQ.                                                          */
/*                                                                          */
/* Command is used to indicate the request that was made of the command     */
/* server in order that we can process the replies appropriately.           */
/*                                                                          */
/* There will be one message on the ReplyToQ for each defined object of     */
/* the requested type. The header of each message will indicate whether     */
/* or not it is the last reply. This procedure will run until all replies   */
/* have been received from the command server.                              */
/*                                                                          */
/* An individual Reply message consists of a header followed by a           */
/* number a parameters, the exact number, type and order will depend        */
/* upon the type of request.                                                */
/*                                                                          */
/* ************************************************************************ */
MQLONG ProcessReplyQ( MQHCONN   hConn
                  , MQHOBJ    hReplyQ
                  , int *     pUserVersion
                  , int *     pQmgrVersion
                  , FILE **poutputfp   /* v3.3 pointer to pointer to the  output file  */
                  , char * fileName   /* v32 Name of the output file  */
                  , MQLONG ObjectSets        /* number of sets of objects to get */
                  , int    SysObjs    /* [v51] System Object Flags */
                  )
{
  MQLONG               AdminMsgLen=0;      /* Length of user message buffer   */
  MQBYTE              *pAdminMsg=NULL;     /* Ptr to outbound data buffer     */
  MQCFH               *pPCFHeader=NULL;    /* Ptr to PCF header structure     */
  MQCFST              *pPCFString=NULL;    /* Ptr to PCF string parm block    */
  MQCFSL              *pPCFStringList=NULL;/* Ptr to PCF string parm block    */
  MQCFIN              *pPCFInteger=NULL;   /* Ptr to PCF integer parm block   */
  MQLONG              *pPCFType=NULL;      /* Type field of PCF message parm  */

  QParms               QDefn;            /*                                 */
  ChlParms             ChlDefn;          /*                                 */
  ProcParms            ProcDefn;         /*                                 */
  QMgrParms            QMgrDefn;         /*                                 */

#if defined (MQCMD_INQUIRE_NAMELIST)
  NameLParms          *pNlDefn=NULL;     /* Namelist [v51] pointer            */
#endif

  /* v53 - authinfo object */
#if defined (MQCMD_INQUIRE_AUTH_INFO)
  AuthInfoParms          AuthInfoDefn;     /* Authinfo [v53] structure */
#endif

  short                Index=0;          /* Loop counter                    */
  int                  i=0;              /* V32 loop control */

  char tempfileName[FILENAMESIZE+1];     /* tempory to build filename */

  int  firstChannel =1;
  int  firstQueue   =1;
  int  firstNamelist=1;
  int  firstProcess =1;
  int  firstQMgr    =1;
  int  firstAuthInfo =1;  /* v530 authinfo */

  MQLONG RetCode = 0;
  /* ********************************************************************** */
  /*                                                                        */
  /* The message will be retrieved into a buffer pointed to by              */
  /* pAdminMsg. This buffer is to have been allocated to be large           */
  /* enough to hold all the parameters for a object type we are             */
  /* expecting, indicated by the input parameter 'Command'.                 */
  /*                                                                        */
  /* pPCFHeader is then allocated to point also to the beginning of         */
  /* the buffer and is used to access the PCF header structure. The         */
  /* header contains several fields. The one we are specifically            */
  /* interested in is the ParameterCount. This tells us how many            */
  /* parameters follow the header in the message buffer. There is           */
  /* one parameter for each local queue attribute known by the              */
  /* queue manager.                                                         */
  /*                                                                        */
  /* At this point we do not know the order or type of each parameter       */
  /* block in the buffer, the first MQLONG of each block defines its        */
  /* type; they may be parameter blocks containing either strings or        */
  /* integers.                                                              */
  /*                                                                        */
  /* pPCFType is used initially to point to the first byte beyond the       */
  /* known parameter block. Initially then, it points to the first byte     */
  /* after the PCF header. Subsequently it is incremented by the length     */
  /* of the identified parameter block and therefore points at the          */
  /* next. Looking at the value of the data pointed to by pPCFType we       */
  /* can decide how to process the next group of bytes, either as a         */
  /* string, or an integer.                                                 */
  /*                                                                        */
  /* In this way we parse the message buffer extracting the values of       */
  /* each of the parameters we are interested in.                           */
  /*                                                                        */
  /* ********************************************************************** */


  /* clear out all fields in the storage arrays */
  memset( &QDefn, '\0', sizeof(QDefn));
  memset( &ChlDefn, '\0', sizeof(ChlDefn));
  ChlDefn.KAInt = -999;   /* special value to indicate not to set KAINT */  
  memset( &ProcDefn, '\0', sizeof(ProcDefn));
  memset( &QMgrDefn, '\0', sizeof(QMgrDefn));

#if defined (MQCMD_INQUIRE_AUTH_INFO)
  memset( &AuthInfoDefn, '\0', sizeof(AuthInfoDefn)); /* v530 */
#endif

#if defined (MQCMD_INQUIRE_NAMELIST)

  /* [v51] becuase namelist take a lot of storage (256*48 bytes */
  /* we'll  malloc the storage instead of getting from stack   */

  if (!(pNlDefn = malloc(sizeof(NameLParms)))){
    fprintf(stderr, "Failure to allocate storage for namelist!\n");
    return (ERROR);
  }
#endif

  /* AdminMsgLen is to be set to the length of the expected reply       */
  /* message. If INT_BUFSZE turns out to be too small, GetMsg will      */
  /* automatically allocate a larger buffer for us                      */

  AdminMsgLen = INT_BUFSZE;
  if(!(pAdminMsg = (MQBYTE *)malloc( AdminMsgLen ))){
    fprintf(stderr, "Failure to allocate %ld bytes for input buffer!\n",
            AdminMsgLen);
    return (ERROR);
  }

   do {
     RetCode = GetMsg(  hConn                      /* Queue manager handle */
                      , hReplyQ                    /* Get queue handle     */
                      , (MQBYTE **)&pAdminMsg      /* pointer to message area*/
                      , &AdminMsgLen
                      );

     if (RetCode) break;

     /* Point to Header */
     pPCFHeader = (MQCFH *)pAdminMsg;

     if (pPCFHeader->Reason) {

       /* v513 ... some platforms return an error if no client channels */
       /* exist, so we'll put out a warning message instead of the      */
       /* more generic pcf error message.                               */

       if (pPCFHeader->Reason==MQRCCF_CHANNEL_NOT_FOUND) {
         if (noisy)   fprintf(stderr, "No client channels found\n");
       } else
       if (pPCFHeader->Reason==MQRC_OBJECT_DAMAGED && ignoreDamaged) {
         fprintf(stderr, "Ignoring 'damaged object' PCF error messsage\n");
         if (*poutputfp) {
           fprintf(*poutputfp, "* ===== \n");
           fprintf(*poutputfp, "* ==>>> Ignoring 'damaged object' PCF error messsage\n");
           fprintf(*poutputfp, "* ===== \n");
         }
       } else

	 /* v530a */
	 /* okay...here's another hack, it appears that if you have NO process definitions */
	 /* the cmd server returns a pcf message with MQRC_UNKNOWN_OBJECT_NAME!            */
	 /* I haven't tested it, but its probably the case with other object types as well */
	 /* so we'll add this to our list of ignorable error codes                         */

       if (pPCFHeader->Reason==MQRC_UNKNOWN_OBJECT_NAME && ignoreDamaged) {
         fprintf(stderr, "Ignoring 'unknown object' PCF error messsage\n");
         if (*poutputfp) {
           fprintf(*poutputfp, "* ===== \n");
           fprintf(*poutputfp, "* ==>>> Ignoring 'unknown object' PCF error messsage\n");
           fprintf(*poutputfp, "* ===== \n");
         }
       } else
       if (pPCFHeader->Reason==MQRCCF_COMMAND_FAILED && ignoreDamaged) {
         fprintf(stderr, "Ignoring 'command failed' PCF error messsage\n");
         if (*poutputfp) {
           fprintf(*poutputfp, "* ===== \n");
           fprintf(*poutputfp, "* ==>>> Ignoring 'command failed' PCF error messsage\n");
           fprintf(*poutputfp, "* ===== \n");
         }
       } else
       {   /* other than channel not found error */
         fprintf(stderr, "Got bad PCF response message\n");
         fprintf( stderr, "\tType = %ld\n", pPCFHeader -> Type);
         fprintf( stderr, "\tStrucLength = %ld\n", pPCFHeader -> StrucLength);
         fprintf( stderr, "\tVersion = %ld\n", pPCFHeader -> Version);
         fprintf( stderr, "\tCommand = %ld\n", pPCFHeader -> Command);
         fprintf( stderr, "\tMsgSeqNumber = %ld\n", pPCFHeader -> MsgSeqNumber);
         fprintf( stderr, "\tControl = %ld\n", pPCFHeader -> Control);
         fprintf( stderr, "\tCompCode = %ld\n", pPCFHeader -> CompCode);
         fprintf( stderr, "\tReason = %ld\n", pPCFHeader -> Reason);
         fprintf( stderr, "\tParameterCount = %ld\n", pPCFHeader -> ParameterCount);

         RetCode = ERROR;
       }  /* end of else other than channel not found error */
     } else {
       /* Point to first parameter */
       pPCFType = (MQLONG *)(pAdminMsg + MQCFH_STRUC_LENGTH);
       Index = 1;

       while ( Index <= pPCFHeader->ParameterCount ) {
        /* Establish the type of each parameter and allocate  */
        /* a pointer of the correct type to reference it.     */
        switch ( *pPCFType ) {
        case MQCFT_INTEGER:
           pPCFInteger = (MQCFIN *)pPCFType;

           switch ( pPCFHeader->Command ) {

           case MQCMD_INQUIRE_Q_MGR:
              ProcessQMgrIntegerParm( pPCFInteger, &QMgrDefn, &RetCode );
              /* [wms] at this time, we have the cmd level of the qmgr */
              *pQmgrVersion = QMgrDefn.CommandLevel;
              break;

           case MQCMD_INQUIRE_Q:
              ProcessQueueIntegerParm( pPCFInteger, &QDefn, &RetCode );
              break;

           case MQCMD_INQUIRE_CHANNEL:
              ProcessChannelIntegerParm( pPCFInteger, &ChlDefn, &RetCode );
              break;

           case MQCMD_INQUIRE_PROCESS:
              ProcessProcIntegerParm( pPCFInteger, &ProcDefn, &RetCode );
              break;

           /* [v51] namelist support */
           #if defined (MQCMD_INQUIRE_NAMELIST)
           case MQCMD_INQUIRE_NAMELIST:
              ProcessNamelistIntegerParm( pPCFInteger, pNlDefn, &RetCode );
              break;
           #endif

           /* [v53] authinfo support */
           #if defined (MQCMD_INQUIRE_AUTH_INFO)
           case MQCMD_INQUIRE_AUTH_INFO:
              ProcessAuthInfoIntegerParm( pPCFInteger, &AuthInfoDefn, &RetCode );
              break;
           #endif

           } /* endswitch */

           /* Increment the pointer to the next parameter by the */
           /* length of the current parm.                        */
           pPCFType = (MQLONG *)( (MQBYTE *)pPCFType
                                + pPCFInteger->StrucLength
                                );
           break;

        case MQCFT_STRING:
           pPCFString = (MQCFST *)pPCFType;
           switch ( pPCFHeader->Command) {

           case MQCMD_INQUIRE_Q_MGR:
              ProcessQMgrStringParm( pPCFString, &QMgrDefn, &RetCode );
              break;

           case MQCMD_INQUIRE_Q:
              ProcessQueueStringParm( pPCFString, &QDefn, &RetCode );
              break;

           case MQCMD_INQUIRE_CHANNEL:
              ProcessChannelStringParm( pPCFString, &ChlDefn, &RetCode );
              break;

           case MQCMD_INQUIRE_PROCESS:
              ProcessProcStringParm( pPCFString, &ProcDefn, &RetCode );
              break;

           /* [v51] namelist support */
           #if defined (MQCMD_INQUIRE_NAMELIST)
           case MQCMD_INQUIRE_NAMELIST:
              ProcessNamelistStringParm( pPCFString, pNlDefn, &RetCode );
              break;
           #endif

           /* [v53] authinfo support */
           #if defined (MQCMD_INQUIRE_AUTH_INFO)
           case MQCMD_INQUIRE_AUTH_INFO:
              ProcessAuthInfoStringParm( pPCFString, &AuthInfoDefn, &RetCode );
              break;
           #endif



           } /* endswitch */

           /* Increment the pointer to the next parameter by the */
           /* length of the current parm.                        */
           pPCFType = (MQLONG *)( (MQBYTE *)pPCFType
                                + pPCFString->StrucLength
                                );

           break;

        /* [v51] process string lists (for namelist) */
        case MQCFT_STRING_LIST:

           pPCFStringList = (MQCFSL *)pPCFType;
           switch ( pPCFHeader->Command ) {

           /* [v51] namelist support */
           #if defined (MQCMD_INQUIRE_NAMELIST)
           case MQCMD_INQUIRE_NAMELIST:
              ProcessNamelistStringListParm(pPCFStringList, pNlDefn, &RetCode );
           break;
           #endif


           /* [v512] include string list for channel exits */
           case MQCMD_INQUIRE_CHANNEL:
              ProcessChannelStringListParm(pPCFStringList, &ChlDefn, &RetCode );
           break;

           default:
               if (noisy) fprintf(stderr, "Got an unexpected StringListParm for %ld\n",
                     pPCFHeader->Command );
               RetCode = ERROR;

           /* Increment the pointer to the next parameter by the */
           /* length of the current parm.                        */
           }

           pPCFType = (MQLONG *)( (MQBYTE *)pPCFType
                                + pPCFStringList->StrucLength
                                );

        break;

        default:
           /* Parameter block of unrecognised type                 */
           /* Something has gone wrong!                            */
           /* Reply message may be corrupt or may be of unexpected */
           /*  format. Either way, continuing may give disasterous */
           /*  results.                                            */
           /* So, terminate program.                               */
           fprintf(stderr,
              "Error: Received parameter block is of an unrecognised type.\n" );
           RetCode = WARNING;
           break;

        } /* endswitch */

        Index++;

     } /* endwhile */

     /* ********************************************************* */
     /* Message parsed, append to output file                     */
     /* ********************************************************* */

     /* V3.2 if filename is not set, set it to the qmgr name + .mqs    */

     if (!fileName[0]) {
        memset(tempfileName, '\0', FILENAMESIZE+1);
        memcpy(tempfileName, QMgrDefn.QMgrName, MQ_Q_MGR_NAME_LENGTH);
        /* replace blanks with nulls and mangle periods and slashes */
        for (i=0; i < MQ_Q_MGR_NAME_LENGTH; i++) {
           if (tempfileName[i] == ' ') tempfileName[i] = '\0';
           if (tempfileName[i] == '.') tempfileName[i] = '!';
           if (tempfileName[i] == '/') tempfileName[i] = '&';
        }  /* end for ... */

        /* Now build the real filename */
        strcpy(fileName, DEFAULTOUTPUTPREFIX);
        strcat(fileName, tempfileName);
        strcat(fileName, DEFAULTOUTPUTPOSTFIX);

     }/* End if !fileName ... */

     /* open file here, once... (if not stdout and not already open) v3.3 */
     if (!*poutputfp) {
        if ((*poutputfp = fopen(fileName, OPENMODE)) == NULL) {
           char buf[1024];
           sprintf(buf, "error opening %s", fileName);
           perror(buf);
           RetCode = ERROR;
           break;
        }
     }

     /* [v51]we are using stdout, so fixup filename */
     if (*poutputfp==stdout) strcpy(fileName, "<stdout>");

    /* based on wether or not the user requested, we'll    */

    /* send the correct qmgr level to the subroutines            */
    if (!(*pUserVersion)) *pUserVersion = *pQmgrVersion;



/* v32 fileName is the name of the output file  */
   switch ( pPCFHeader->Command) {

/* [v51] namelist support */

      #if defined (MQCMD_INQUIRE_NAMELIST)
      case MQCMD_INQUIRE_NAMELIST:
         /* Only write if name is NOT "SYSTEM." or user wants them */
         if (memcmp(pNlDefn->NamelistName, "SYSTEM.", 7) || SysObjs) {
             if (firstNamelist) {
                firstNamelist = 0;
                fprintf(stderr, "Namelist definitions written to %.*s.\n",
                        FILENAMESIZE, fileName );
             }/* End if*/
             AddToFileNAMELIST( pNlDefn, *pUserVersion, fileName,
                                *poutputfp, &RetCode );
         }/* End if*/
         break;
      #endif

      case MQCMD_INQUIRE_Q:
         /* Only write if name is NOT "SYSTEM." or user wants them */
         if (memcmp(QDefn.QName, "SYSTEM.", 7) || SysObjs) {
            if (firstQueue) {
               firstQueue = 0;
               fprintf(stderr, "Queue definitions written to %.*s.\n",
                       FILENAMESIZE, fileName );
            }/* End if*/
            AddToFileQUEUE( &QDefn, *pUserVersion, fileName,
                            *poutputfp, &RetCode );
         }
         break;

      case MQCMD_INQUIRE_CHANNEL:
         /* Only write if name is NOT "SYSTEM." or user wants them */
         if (memcmp(ChlDefn.ChannelName, "SYSTEM.", 7) || SysObjs) {
            if (firstChannel) {
               firstChannel = 0;
               fprintf(stderr, "Channel definitions written to %.*s.\n",
                       FILENAMESIZE, fileName );
              }/* End if*/
            AddToFileCHANNEL( &ChlDefn, *pUserVersion, fileName,
                              *poutputfp, &RetCode );
         }
         break;

      case MQCMD_INQUIRE_PROCESS:
         /* Only write if name is NOT "SYSTEM." or user wants them */
         if (memcmp(ProcDefn.ProcessName, "SYSTEM.", 7) || SysObjs) {
            if (firstProcess) {
               firstProcess = 0;
               fprintf(stderr, "Process definitions written to %.*s.\n",
                       FILENAMESIZE, fileName );
              }/* End if*/
            AddToFilePROCESS( &ProcDefn, *pUserVersion, fileName,
                              *poutputfp, &RetCode );
         }
         break;

      case MQCMD_INQUIRE_Q_MGR:
         if (firstQMgr) {
            firstQMgr = 0;
            fprintf(stderr, "Queue Manager definitions written to %.*s.\n",
                    FILENAMESIZE, fileName );
           }/* End if*/
         AddToFileQMGR( &QMgrDefn, *pUserVersion, fileName,
                        *poutputfp, &RetCode );
         break;

/* [v53] authinfo support */

      #if defined (MQCMD_INQUIRE_AUTH_INFO)
      case MQCMD_INQUIRE_AUTH_INFO:
         /* Only write if name is NOT "SYSTEM." or user wants them */
         if (memcmp(AuthInfoDefn.AuthInfoName, "SYSTEM.", 7) || SysObjs) {
             if (firstAuthInfo) {
                firstAuthInfo = 0;
                fprintf(stderr, "AuthInfo definitions written to %.*s.\n",
                        FILENAMESIZE, fileName );
             }/* End if*/
             AddToFileAUTHINFO( &AuthInfoDefn, *pUserVersion, fileName,
                                *poutputfp, &RetCode );
         }/* End if*/
         break;
      #endif




      } /* endswitch */

      if (RetCode==ERROR) {
        fprintf(stderr, "Error writing to ouput file %s\n", fileName);
      }/* End if*/


      /* ********************************************************* */
      /* Finished processing the current message, do the next one. */
      /* ********************************************************* */

      /* ********************************************************* */
      /* If we hit a "last message" message, reduce the number of  */
      /* objectsets by 1 until none are left                       */
      /* ********************************************************* */

      if (pPCFHeader->Control == MQCFC_LAST) ObjectSets--;

     }   /* end of else pcf rc is okay */

   } while ( RetCode != ERROR && ObjectSets ); /* enddo */

   if (pAdminMsg)
      free( pAdminMsg );

   #if defined (MQCMD_INQUIRE_NAMELIST)
   if (pNlDefn)
      free( pNlDefn );
   #endif

   return (RetCode);
}



